<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Services\Slug;
use App\User;
use App\Location;
use App\LocationPermission;
use App\WareHouse;
use App\WareHouseTrack;

class WareHouseController extends Controller
{

    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('warehouse-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $warehouses = WareHouse::Where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.masters.warehouses.index', compact('warehouses'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $user = Auth::user();
        if (!$user->can('warehouse-master')) {
            abort(403, 'Unauthorized action.');
        }
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        return view('user.masters.warehouses.create', compact('users'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();

        $warehouse = Warehouse::where('name', $request->name)->where('location_id', $location_id)->first();
        if($warehouse){
            Session::flash('alert-danger', 'This Warehouse Already Exists');
            return redirect()->back();
        }
        else{
            $warehouse = new WareHouse();
            $warehouse->location_id = $location_id;
            $warehouse->name = $request->name;
            $warehouse->slug = $this->slug->createSlug('warehouse', $request->name);
            // $warehouse->show_user = $request->show_user;
            $warehouse->status = 'active';
            $warehouse->save();

            if(isset($request->show_user)){
                foreach ($request->show_user as $key => $request->show_user) {
                    $ware_house_track = new WareHouseTrack();
                    $ware_house_track->user_id = $request->show_user;
                    $ware_house_track->ware_house_id = $warehouse->id;
                    $ware_house_track->save();
                }
            }

            if(Auth::user()->hasRole('super-admin'))
            {
                $ware_house_track = new WareHouseTrack();
                $ware_house_track->user_id = Auth::user()->id;
                $ware_house_track->ware_house_id = $warehouse->id;
                $ware_house_track->save();
            }else{
                $admins = User::role('super-admin')->get();
                foreach ($admins as $key => $admin) {
                    $ware_house_track = new WareHouseTrack();
                    $ware_house_track->user_id = $admin->id;
                    $ware_house_track->ware_house_id = $warehouse->id;
                    $ware_house_track->save();
                }
                $ware_house_track = new WareHouseTrack();
                $ware_house_track->user_id = Auth::user()->id;
                $ware_house_track->ware_house_id = $warehouse->id;
                $ware_house_track->save();
            }

            Session::flash('alert-success', 'WareHouse created successfully');
            return redirect('/masters/warehouses');
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\WareHouse  $warehouse
     * @return \Illuminate\Http\Response
     */
    public function show(WareHouse $warehouse)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\WareHouse  $warehouse
     * @return \Illuminate\Http\Response
     */
    public function edit($warehouse_id)
    {
        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $user = Auth::user();
        if (!$user->can('warehouse-master')) {
            abort(403, 'Unauthorized action.');
        }

        $warehouse = WareHouse::find($warehouse_id);
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $ware_house_track = WareHouseTrack::Where('ware_house_id', $warehouse_id)->pluck('user_id');
        return view('user.masters.warehouses.edit', compact('warehouse', 'users', 'ware_house_track'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\WareHouse  $warehouse
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $warehouse_id)
    {
        $warehouse = WareHouse::find($warehouse_id);
        $warehouse->name = $request->name;
        $warehouse->slug = $this->slug->createSlug('warehouse', $request->name);
        // $warehouse->show_user = $request->show_user;
        $warehouse->status = 'active';
        $warehouse->save();

        WareHouseTrack::where('ware_house_id', $warehouse_id)->delete();

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $ware_house_track = new WareHouseTrack();
                $ware_house_track->user_id = $request->show_user;
                $ware_house_track->ware_house_id = $warehouse_id;
                $ware_house_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $ware_house_track = new WareHouseTrack();
            $ware_house_track->user_id = Auth::user()->id;
            $ware_house_track->ware_house_id = $warehouse->id;
            $ware_house_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $ware_house_track = new WareHouseTrack();
                $ware_house_track->user_id = $admin->id;
                $ware_house_track->ware_house_id = $warehouse->id;
                $ware_house_track->save();
            }
            $ware_house_track = new WareHouseTrack();
            $ware_house_track->user_id = Auth::user()->id;
            $ware_house_track->ware_house_id = $warehouse->id;
            $ware_house_track->save();
        }

        Session::flash('alert-success', 'WareHouse updated successfully');
        return redirect('/masters/warehouses');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\WareHouse  $warehouse
     * @return \Illuminate\Http\Response
     */
    public function destroy($warehouse_id)
    {
        $warehouse = WareHouse::findOrFail($warehouse_id);
        $warehouse->delete();

        WareHouseTrack::where('ware_house_id', $warehouse_id)->delete();

        Session::flash('alert-success', 'WareHouse deleted successfully');
        return redirect('/masters/warehouses');
    }

    public function softdelete($warehouse_id)
    {
        $warehouse = WareHouse::findOrFail($warehouse_id);
        $warehouse->forceDelete();

        WareHouseTrack::where('ware_house_id', $warehouse_id)->forceDelete();

        Session::flash('alert-success', 'WareHouse deleted successfully');
        return redirect('/masters/warehouses');
    }
}
